<?php

namespace Drupal\spammaster\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class controller.
 */
class SpamMasterCleanUpController extends ControllerBase {

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Logger service.
   *
   * @var Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $configFactory) {
    $this->connection = $connection;
    $this->loggerFactory = $logger_factory;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('logger.factory'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spammastercleanupkeys() {

    // Get variables.
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license_status = $spammaster_settings->get('spammaster.license_status');
    if ($spammaster_license_status == 'VALID' || $spammaster_license_status == 'MALFUNCTION_1' || $spammaster_license_status == 'MALFUNCTION_2') {
      // Prepare now date.
      $spammaster_date = date("Y-m-d H:i:s");

      // Process system log.
      $spammaster_cleanup_system = $spammaster_settings->get('spammaster.cleanup_system');
      if ($spammaster_cleanup_system == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: System weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: System weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_sys_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_system.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :system AND date <= :time_expires', [':system' => 'spammaster', ':time_expires' => $minus_sys_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: System weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_system . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: System weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_system . ' months.',
        ])->execute();
      }
      // Process cron log.
      $spammaster_cleanup_cron = $spammaster_settings->get('spammaster.cleanup_cron');
      if ($spammaster_cleanup_cron == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Cron weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Cron weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_cron_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_cron.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :cron AND date <= :time_expires', [':cron' => 'spammaster-cron', ':time_expires' => $minus_cron_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Cron weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_cron . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Cron weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_cron . ' months.',
        ])->execute();
      }

      // Process mail log.
      $spammaster_cleanup_mail = $spammaster_settings->get('spammaster.cleanup_mail');
      if ($spammaster_cleanup_mail == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Mail weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Mail weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_mail_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_mail.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :mail AND date <= :time_expires', [':mail' => 'spammaster-mail', ':time_expires' => $minus_mail_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Mail weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_mail . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Mail weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_mail . ' months.',
        ])->execute();
      }

      // Process whitelist log.
      $spammaster_cleanup_whitelist = $spammaster_settings->get('spammaster.cleanup_whitelist');
      if ($spammaster_cleanup_whitelist == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Whitelist weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Whitelist weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_white_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_whitelist.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :white AND date <= :time_expires', [':white' => 'spammaster-whitelist', ':time_expires' => $minus_white_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Whitelist weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_whitelist . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Whitelist weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_whitelist . ' months.',
        ])->execute();
      }

      // Process firewall log.
      $spammaster_cleanup_firewall = $spammaster_settings->get('spammaster.cleanup_firewall');
      if ($spammaster_cleanup_firewall == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Firewall weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Firewall weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_fire_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_firewall.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :fire AND date <= :time_expires', [':fire' => 'spammaster-firewall', ':time_expires' => $minus_fire_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Firewall weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_firewall . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Firewall weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_firewall . ' months.',
        ])->execute();
      }

      // Process registration log.
      $spammaster_cleanup_registration = $spammaster_settings->get('spammaster.cleanup_registration');
      if ($spammaster_cleanup_registration == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Registration weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Registration weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_reg_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_registration.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :reg AND date <= :time_expires', [':reg' => 'spammaster-registration', ':time_expires' => $minus_reg_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Registration weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_registration . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Registration weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_registration . ' months.',
        ])->execute();
      }

      // Process comment log.
      $spammaster_cleanup_comment = $spammaster_settings->get('spammaster.cleanup_comment');
      if ($spammaster_cleanup_comment == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Comment weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Comment weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_com_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_comment.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :com AND date <= :time_expires', [':com' => 'spammaster-comment', ':time_expires' => $minus_com_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Comment weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_comment . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Comment weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_comment . ' months.',
        ])->execute();
      }

      // Process contact log.
      $spammaster_cleanup_contact = $spammaster_settings->get('spammaster.cleanup_contact');
      if ($spammaster_cleanup_contact == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Contact weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Contact weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_con_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_contact.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :con AND date <= :time_expires', [':con' => 'spammaster-contact', ':time_expires' => $minus_con_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Contact weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_contact . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Contact weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_contact . ' months.',
        ])->execute();
      }

      // Process honeypot log.
      $spammaster_cleanup_honeypot = $spammaster_settings->get('spammaster.cleanup_honeypot');
      if ($spammaster_cleanup_honeypot == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Honeypot weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Honeypot weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_hon_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_honeypot.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :hon AND date <= :time_expires', [':hon' => 'spammaster-honeypot', ':time_expires' => $minus_hon_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Honeypot weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_honeypot . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: Honeypot weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_honeypot . ' months.',
        ])->execute();
      }

      // Process recaptcha log.
      $spammaster_cleanup_recaptcha = $spammaster_settings->get('spammaster.cleanup_recaptcha');
      if ($spammaster_cleanup_recaptcha == '0') {
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: reCaptcha weekly clean-up cron did not run. Is set to infinite.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: reCaptcha weekly clean-up cron did not run. Is set to infinite.',
        ])->execute();
      }
      else {
        $minus_rec_months_time = date('Y-m-d h:i:s', strtotime("-'.$spammaster_cleanup_recaptcha.' months", strtotime($spammaster_date)));
        // Delete data older than 3 months.
        $this->connection->query('DELETE FROM {spammaster_keys} WHERE spamkey = :rec AND date <= :time_expires', [':rec' => 'spammaster-recaptcha', ':time_expires' => $minus_rec_months_time]);
        // Log and watchdog.
        $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: reCaptcha weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_recaptcha . ' months.');
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-cron',
          'spamvalue' => 'Spam Master: reCaptcha weekly clean-up cron successfully run. Deleted older than ' . $spammaster_cleanup_recaptcha . ' months.',
        ])->execute();
      }
    }
    else {
      // Log message.
      $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Weekly logs clean-up cron did not run, check your license status.');
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $spammaster_date,
        'spamkey' => 'spammaster-cron',
        'spamvalue' => 'Spam Master: Weekly logs clean-up cron did not run, check your license status.',
      ])->execute();
    }

  }

  /**
   * {@inheritdoc}
   */
  public function spammastercleanupbuffer() {

    // Get variables.
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license_status = $spammaster_settings->get('spammaster.license_status');
    if ($spammaster_license_status == 'VALID' || $spammaster_license_status == 'MALFUNCTION_1' || $spammaster_license_status == 'MALFUNCTION_2') {
      // Set 90 days time.
      $time = date('Y-m-d H:i:s');
      $time_expires = date('Y-m-d H:i:s', strtotime($time . '-90 days'));

      // Delete data older than 3 months.
      $this->connection->query('DELETE FROM {spammaster_threats} WHERE date <= :time_expires', [':time_expires' => $time_expires]);

      // Log message.
      $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Buffer weekly clean-up cron successful run.');
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $time,
        'spamkey' => 'spammaster-cron',
        'spamvalue' => 'Spam Master:  Buffer weekly clean-up cron successful run.',
      ])->execute();
    }
    else {
      // Log message.
      $this->loggerFactory->get('spammaster-cron')->notice('Spam Master:  Buffer weekly clean-up cron did run, check your license status.');
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $time,
        'spamkey' => 'spammaster-cron',
        'spamvalue' => 'Spam Master:  Buffer weekly clean-up cron did not run, check your license status.',
      ])->execute();
    }

  }

}
