<?php

namespace Drupal\spammaster\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use GuzzleHttp\Client;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class controller.
 */
class SpamMasterCommentController extends ControllerBase {

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Logger service.
   *
   * @var Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $form;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $formstate;

  /**
   * The spammastercommentcheck class.
   *
   * @var \Drupal\spammmaster\spammastercommentcheck
   */
  protected $spammasterip;

  /**
   * The spammastercommentcheck class.
   *
   * @var \Drupal\spammmaster\spammastercommentcheck
   */
  protected $spammasteremail;

  /**
   * The spammastercommentcheck class.
   *
   * @var \Drupal\spammmaster\spammastercommentcheck
   */
  protected $spammastercomment;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, LoggerChannelFactoryInterface $logger_factory, MessengerInterface $messenger, RequestStack $request_stack, Client $httpClient, ConfigFactoryInterface $configFactory) {
    $this->connection = $connection;
    $this->loggerFactory = $logger_factory;
    $this->messenger = $messenger;
    $this->requestStack = $request_stack;
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('logger.factory'),
      $container->get('messenger'),
      $container->get('request_stack'),
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spammastercommentcheck($form, $formstate, $spammasterip, $spammasteremail, $spammastercomment) {

    $this->form = $form;
    $this->formstate = $formstate;
    $this->spammasterip = $spammasterip;
    $this->spammasteremail = $spammasteremail;
    $this->spammastercomment = $spammastercomment;
    $spammaster_date = date("Y-m-d H:i:s");
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license = $spammaster_settings->get('spammaster.license_key');
    $spammaster_status = $spammaster_settings->get('spammaster.license_status');
    $spammaster_total_block_count = $spammaster_settings->get('spammaster.total_block_count');
    $spammaster_settings_protection = $this->configFactory->getEditable('spammaster.settings_protection');
    $spammaster_block_message = $spammaster_settings_protection->get('spammaster.block_message');
    $blog_threat_ip = $this->requestStack->getCurrentRequest()->getClientIp();
    if ($spammaster_status == 'VALID' || $spammaster_status == 'MALFUNCTION_1' || $spammaster_status == 'MALFUNCTION_2') {
      // Exempt whitelist.
      $spammaster_white_query = $this->connection->select('spammaster_white', 'u');
      $spammaster_white_query->fields('u', ['white']);
      $spammaster_white_query->where('(white = :ip OR white = :email)', [':ip' => $spammasterip, ':email' => $spammasteremail]);
      $spammaster_white_result = $spammaster_white_query->execute()->fetchObject();
      // White positive, watchdog, insert.
      if (!empty($spammaster_white_result)) {
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-comment',
          'spamvalue' => 'Spam Master: comment found in whitelist, Ip: ' . $spammasterip . ', Email: ' . $spammasteremail,
        ])->execute();

        $this->loggerFactory->get('spammaster-comment')->notice('Spam Master: comment found in whitelist, Ip: ' . $spammasterip . ', Email: ' . $spammasteremail);
      }
      else {
        // Local db check.
        $spammaster_spam_buffer_query = $this->connection->select('spammaster_threats', 'u');
        $spammaster_spam_buffer_query->fields('u', ['threat']);
        $spammaster_spam_buffer_query->where('(threat = :ip OR threat = :email)', [':ip' => $spammasterip, ':email' => $spammasteremail]);
        $spammaster_spam_buffer_result = $spammaster_spam_buffer_query->execute()->fetchObject();
        $result_comment_content_trim = substr($spammastercomment, 0, 360);
        $result_comment_content_clean = strip_tags($result_comment_content_trim);
        if (empty($result_comment_content_clean)) {
          $result_comment_content_clean = 'your-comment';
        }
        // Local db positive, throw error, watchdog, insert.
        if (!empty($spammaster_spam_buffer_result)) {
          $formstate->setErrorByName('mail', 'SPAM MASTER: ' . $spammaster_block_message);

          // Insert local db.
          $spammaster_db_ip = $this->connection->select('spammaster/Dr_threats', 'u');
          $spammaster_db_ip->fields('u', ['threat']);
          $spammaster_db_ip->where('(threat = :ip)', [':ip' => $spammasterip]);
          $spammaster_db_ip_result = $spammaster_db_ip->execute()->fetchObject();
          if (empty($spammaster_db_ip_result)) {
            $this->connection->insert('spammaster_threats')->fields([
              'date' => $spammaster_date,
              'threat' => $spammasterip,
            ])->execute();
          }

          $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
          $this->configFactory->getEditable('spammaster.settings')
            ->set('spammaster.total_block_count', $spammaster_total_block_count_1)
            ->save();

          $this->connection->insert('spammaster_keys')->fields([
            'date' => $spammaster_date,
            'spamkey' => 'spammaster-comment',
            'spamvalue' => 'Spam Master: comment buffer block, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean,
          ])->execute();

          $this->loggerFactory->get('spammaster-comment')->notice('Spam Master: buffer comment block, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean);
        }
        // Web api check.
        else {
          // Create data to be posted.
          $blog_license_key = $spammaster_license;
          $blog_threat_type = 'comment';
          if (empty($spammasteremail) || is_array($spammasteremail)) {
            $spammasteremail = 'drup@' . date('Ymdhis') . '.drup';
          }
          $blog_threat_email = $spammasteremail;
          $blog_web_address = $this->requestStack->getCurrentRequest()->getHost();
          $address_unclean = $blog_web_address;
          $address = preg_replace('#^https?://#', '', $address_unclean);
          $blog_server_ip = $_SERVER['SERVER_ADDR'];
          // If empty ip.
          if (empty($blog_server_ip) || $blog_server_ip == '0') {
            @$blog_server_ip = 'I ' . gethostbyname($_SERVER['SERVER_NAME']);
          }
          $spam_master_leaning_url = 'aHR0cHM6Ly9zcGFtbWFzdGVyLnRlY2hnYXNwLmNvbS93cC1jb250ZW50L3BsdWdpbnMvc3BhbS1tYXN0ZXItYWRtaW5pc3RyYXRvci9pbmNsdWRlcy9sZWFybmluZy9nZXRfbGVhcm5fY29tLnBocA==';
          // Call drupal hhtpclient.
          $client = $this->httpClient;
          // Post data.
          $request = $client->post(base64_decode($spam_master_leaning_url), [
            'form_params' => [
              'blog_license_key' => $blog_license_key,
              'blog_threat_ip' => $blog_threat_ip,
              'blog_threat_type' => $blog_threat_type,
              'blog_threat_email' => $blog_threat_email,
              'blog_threat_content' => $result_comment_content_clean,
              'blog_web_adress' => $address,
              'blog_server_ip' => $blog_server_ip,
            ],
          ]);
          // Decode json data.
          $response = json_decode($request->getBody(), TRUE);
          if (empty($response)) {
            // Log clean rbl check.
            $this->connection->insert('spammaster_keys')->fields([
              'date' => $spammaster_date,
              'spamkey' => 'spammaster-comment',
              'spamvalue' => 'Spam Master: comment delivered, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean,
            ])->execute();

            $this->loggerFactory->get('spammaster-comment')->notice('Spam Master: comment delivered, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean);
          }
          else {
            // Insert local db.
            $spammaster_db_ip = $this->connection->select('spammaster_threats', 'u');
            $spammaster_db_ip->fields('u', ['threat']);
            $spammaster_db_ip->where('(threat = :ip)', [':ip' => $spammasterip]);
            $spammaster_db_ip_result = $spammaster_db_ip->execute()->fetchObject();
            if (empty($spammaster_db_ip_result)) {
              $this->connection->insert('spammaster_threats')->fields([
                'date' => $spammaster_date,
                'threat' => $spammasterip,
              ])->execute();
            }
            $spammaster_db_email = $this->connection->select('spammaster_threats', 'u');
            $spammaster_db_email->fields('u', ['threat']);
            $spammaster_db_email->where('(threat = :email)', [':email' => $spammasteremail]);
            $spammaster_db_email_result = $spammaster_db_email->execute()->fetchObject();
            if (empty($spammaster_db_email_result)) {
              $this->connection->insert('spammaster_threats')->fields([
                'date' => $spammaster_date,
                'threat' => $spammasteremail,
              ])->execute();
            }
            // Web positive, throw error.
            $formstate->setErrorByName('mail', 'SPAM MASTER: ' . $spammaster_block_message);
            // Watchdog log.
            $this->loggerFactory->get('spammaster-comment')->notice('Spam Master: comment rbl block, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean);
            $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
            $this->configFactory->getEditable('spammaster.settings')
              ->set('spammaster.total_block_count', $spammaster_total_block_count_1)
              ->save();
            $this->connection->insert('spammaster_keys')->fields([
              'date' => $spammaster_date,
              'spamkey' => 'spammaster-comment',
              'spamvalue' => 'Spam Master: comment rbl block, Ip: ' . $spammasterip . ', Comment: ' . $result_comment_content_clean,
            ])->execute();
          }
        }
      }
    }
  }

}
