<?php

namespace Drupal\spammaster\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class controller.
 */
class SpamMasterCronController extends ControllerBase {

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Logger service.
   *
   * @var Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Lic Controller.
   *
   * @var \Drupal\spammaster\Controller\SpamMasterLicController
   */
  protected $licController;

  /**
   * The Mail Controller.
   *
   * @var \Drupal\spammaster\Controller\SpamMasterMailController
   */
  protected $mailController;

  /**
   * The Clean Controller.
   *
   * @var \Drupal\spammaster\Controller\SpamMasterCleanUpController
   */
  protected $cleanController;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $configFactory, SpamMasterLicController $licController, SpamMasterMailController $mailController, SpamMasterCleanUpController $cleanController) {
    $this->connection = $connection;
    $this->loggerFactory = $logger_factory;
    $this->configFactory = $configFactory;
    $this->SpamMasterLicController = $licController;
    $this->SpamMasterMailController = $mailController;
    $this->SpamMasterCleanUpController = $cleanController;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('spammaster.lic_controller'),
      $container->get('spammaster.mail_controller'),
      $container->get('spammaster.clean_controller')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spammasterdailycron() {
    $spammaster_date = date('Y-m-d H:i:s');
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_response_key = $spammaster_settings->get('spammaster.license_status');
    $spammaster_alert_3 = $spammaster_settings->get('spammaster.license_alert_level');
    $spammaster_settings_protection = $this->configFactory->getEditable('spammaster.settings_protection');
    $spammaster_email_alert_3 = $spammaster_settings_protection->get('spammaster.email_alert_3');
    $spammaster_email_daily_report = $spammaster_settings_protection->get('spammaster.email_daily_report');

    if ($spammaster_response_key == 'VALID' || $spammaster_response_key == 'MALFUNCTION_1' || $spammaster_response_key == 'MALFUNCTION_2') {
      // Implements daily cron request via controllers.
      // Call Lic Controller.
      $spammaster_lic_controller = $this->SpamMasterLicController;
      $spammaster_lic_controller->spammasterlicdaily();

      // Call Mail Controller.
      $spammaster_mail_controller = $this->SpamMasterMailController;
      if ($spammaster_email_alert_3 != 0 && $spammaster_alert_3 == 'ALERT_3') {
        $spammaster_mail_controller->spammasterlicalertlevel3();
      }
      if ($spammaster_email_daily_report != 0) {
        $spammaster_mail_controller->spammastermaildailyreport();
      }
    }
    else {
      // Log message.
      $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Warning! daily cron did run, check your license status.');
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $spammaster_date,
        'spamkey' => 'spammaster-cron',
        'spamvalue' => 'Spam Master: Warning! daily cron did not run, check your license status.',
      ])->execute();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spammasterweeklycron() {
    $spammaster_date = date('Y-m-d H:i:s');
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $response_key = $spammaster_settings->get('spammaster.license_status');
    $spammaster_settings_protection = $this->configFactory->getEditable('spammaster.settings_protection');
    $spammaster_email_weekly_report = $spammaster_settings_protection->get('spammaster.email_weekly_report');
    $spammaster_email_improve = $spammaster_settings_protection->get('spammaster.email_improve');

    if ($response_key == 'VALID' || $response_key == 'MALFUNCTION_1' || $response_key == 'MALFUNCTION_2') {
      // Implements daily cron request via controllers.
      // Call Mail Controller.
      $spammaster_mail_controller = $this->SpamMasterMailController;
      if ($spammaster_email_weekly_report != 0) {
        $spammaster_mail_controller->spammastermailweeklyreport();
      }
      if ($spammaster_email_improve != 0) {
        $spammaster_mail_controller->spammastermailhelpreport();
      }
      $spammaster_cleanup_controller = $this->SpamMasterCleanUpController;
      $spammaster_cleanup_controller->spammastercleanupkeys();
      $spammaster_cleanup_controller->spammastercleanupbuffer();
    }
    else {
      // Log message.
      $this->loggerFactory->get('spammaster-cron')->notice('Spam Master: Warning! weekly cron did run, check your license status.');
      $this->connection->insert('spammaster_keys')->fields([
        'date' => $spammaster_date,
        'spamkey' => 'spammaster-cron',
        'spamvalue' => 'Spam Master: Warning! weekly cron did not run, check your license status.',
      ])->execute();
    }
  }

}
